# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

# VLAD 500.0 300.0 C32/40 B450C 0.85 0KN 193KNm alfa 0 603mm2
import csv
import os
import time

from pycivil.EXAStructural import templateRCRect as est
from pycivil.EXAUtils import codeAster as au

plottingOnScreen = False
markers = False

start = time.time()

errors = []
errors_skipped = []
columnForCSV = []

fileName = os.path.join(
    os.path.dirname(os.path.realpath(est.__file__)), "../tests/csv_sls.resu"
)

tableFerr12 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbferr12", "#PYTHON_END_tbferr12"
)
tableFerr12.parse()
AYI = tableFerr12.fetchColumnWithName("AYI")
AYS = tableFerr12.fetchColumnWithName("AYS")

tableEfge12 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbefge12", "#PYTHON_END_tbefge12"
)
tableEfge12.parse()
N = tableEfge12.fetchColumnWithName("N")
MFY = tableEfge12.fetchColumnWithName("MFY")
MFZ = tableEfge12.fetchColumnWithName("MFZ")

MAILLE12 = tableFerr12.fetchColumnWithName("MAILLE")

areaSteel12 = []
for idx, _v in enumerate(AYI):
    idx
    if AYI[idx] != 0.0:
        areaSteel12.append(AYI[idx] * 1e06)
    if AYS[idx] != 0.0:
        areaSteel12.append(AYS[idx] * 1e06)

MFZList12 = []
NList12 = []
for idx, _v in enumerate(MFZ[: int(len(MFZ) / 2)]):
    MFZList12.append(abs((MFZ[idx * 2] + MFZ[idx * 2 + 1]) / 2.0) * 1e03)
    NList12.append(-(N[idx * 2] + N[idx * 2 + 1]) / 2)

section = None
for idx, _v in enumerate(MFZList12):
    columnCSVName = "section_01-300x500"
    columnCSVPointN = str("%1.2f" % (NList12[idx] * 1e-03))
    columnCSVPointM = str("%1.2f" % (MFZList12[idx] * 1e-06))

    strCase = (
        columnCSVName
        + " - mesh: "
        + MAILLE12[idx]
        + " - "
        + "N="
        + columnCSVPointN
        + " [KN] - "
        + "M="
        + columnCSVPointM
        + " [KN*m]"
    )
    Area, extra, section = est.RCTemplRectEC2DesignNM(
        name=strCase,
        elementType="beam",
        steel="B450C",
        concrete="C32/40",
        sectionH=500.0,
        sectionW=300.0,
        topRecover=40.0,
        botRecover=40.0,
        N=NList12[idx],
        M=MFZList12[idx],
        toll=1e-5,
        toll_bisec=1e-3,
        maxIt=100,
        plot=plottingOnScreen,
        alphaTopBot=0,
        factorStart=0,
        factorEnd=1,
        points=100,
        logLevel=0,
        markersOnDomain=markers,
        savingSingleDomains=False,
        sigmacMax=0.65 * 32,
        sigmasMax=0.80 * 450,
        homogeneization=15.0,
        SLS=True,
        outSection=True,
    )
    columnCSVArea = "%1.3f" % Area
    print("===============")
    print("Case Test string:" + strCase)
    print("Area with bisection or secants: %1.3f" % Area)
    print("Area with FERRAILLAGE 1D: %1.3f" % areaSteel12[idx])

    if extra is not None:
        print("From elastic alghoritm ...")
        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (-extra[5], -extra[6], extra[7])
        )
        section.assignSteelAreaWithAlphaWeight(Area)
        if abs(NList12[idx]) < 0.0000005:
            # print(section)
            extra_analytical = section.solverSLS_M(MFZList12[idx])
            print("From analytical with only M ...")
        else:
            extra_analytical = section.solverSLS_NM(-NList12[idx], MFZList12[idx])
            print("From analytical with N and M ...")

        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (extra_analytical[0], extra_analytical[1], extra_analytical[2])
        )
        error1 = abs((-extra[5] - extra_analytical[0]) / extra_analytical[0])
        # in code_aster compression is positive
        # in python  compression is negative
        error2 = abs((-extra[6] - extra_analytical[1]) / extra_analytical[1])
        error3 = abs((extra[7] - extra_analytical[2]) / extra_analytical[2])
        error_from_analitycal = max(error1, error2, error3)
        print("Error from analytical : %1.3E" % error_from_analitycal)
    else:
        print("Elastic algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Analytical algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Error from analytical : N.D.")

    error = (areaSteel12[idx] - Area) / (Area)
    if not (areaSteel12[idx] == -1000000.0):
        errors.append(abs(error))
    else:
        errors_skipped.append(abs(error))
    print("Error: %1.3e" % error)
    print("===============")
    columnForCSV.append(
        [
            columnCSVName,
            "B450C",
            "C32/40",
            columnCSVPointN,
            columnCSVPointM,
            columnCSVArea,
        ]
    )

tableFerr34 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbferr34", "#PYTHON_END_tbferr34"
)
tableFerr34.parse()
AZI = tableFerr34.fetchColumnWithName("AZI")
AZS = tableFerr34.fetchColumnWithName("AZS")

tableEfge34 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbefge34", "#PYTHON_END_tbefge34"
)
tableEfge34.parse()

N = tableEfge34.fetchColumnWithName("N")
MFY = tableEfge34.fetchColumnWithName("MFY")
MFZ = tableEfge34.fetchColumnWithName("MFZ")

MAILLE34 = tableFerr34.fetchColumnWithName("MAILLE")

areaSteel34 = []
for idx, _v in enumerate(AZI):
    if AZI[idx] != 0.0:
        areaSteel34.append(AZI[idx] * 1e06)
    if AZS[idx] != 0.0:
        areaSteel34.append(AZS[idx] * 1e06)

MFYList34 = []
NList34 = []
for idx, _v in enumerate(MFY[: int(len(MFY) / 2)]):
    # MFYList34.append(abs( ( MFY[idx*2] + MFY[idx*2+1] ) / 2.0 )*1E+03 )
    MFYList34.append(abs((MFY[idx * 2] + MFY[idx * 2 + 1]) / 2.0) * 1e03)
    NList34.append(-(N[idx * 2] + N[idx * 2 + 1]) / 2)

idxerror = []
for idx, _v in enumerate(MFYList34):
    columnCSVName = "section_02-500x300"
    columnCSVPointN = str("%1.2f" % (NList34[idx] * 1e-03))
    columnCSVPointM = str("%1.2f" % (MFYList34[idx] * 1e-06))

    strCase = (
        columnCSVName
        + " - mesh: "
        + MAILLE34[idx]
        + " - N="
        + columnCSVPointN
        + " [KN] - "
        + "M="
        + columnCSVPointM
        + " [KN*m]"
    )
    Area, extra, section = est.RCTemplRectEC2DesignNM(
        name=strCase,
        elementType="beam",
        steel="B450C",
        concrete="C32/40",
        alphacc=1.00,
        sectionH=300.0,
        sectionW=500.0,
        topRecover=40.0,
        botRecover=40.0,
        N=NList34[idx],
        M=MFYList34[idx],
        toll=1e-5,
        toll_bisec=1e-3,
        maxIt=100,
        plot=plottingOnScreen,
        alphaTopBot=0,
        factorStart=0,
        factorEnd=1,
        points=100,
        logLevel=0,
        markersOnDomain=markers,
        savingSingleDomains=False,
        sigmacMax=0.65 * 32,
        sigmasMax=0.80 * 450,
        homogeneization=15.0,
        SLS=True,
        outSection=True,
    )
    columnCSVArea = "%1.3f" % Area
    print("===============")
    print("Case Test string:" + strCase)
    print("Area with bisection or secants: %1.3f" % Area)
    print("Area with FERRAILLAGE 1D: %1.3f" % areaSteel34[idx])

    if extra is not None:
        print("From elastic alghoritm ...")
        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (-extra[5], -extra[6], extra[7])
        )
        section.assignSteelAreaWithAlphaWeight(Area)
        if abs(NList34[idx]) < 0.0000005:
            # print(section)
            extra_analytical = section.solverSLS_M(MFYList34[idx])
            print("From analytical with only M ...")
        else:
            extra_analytical = section.solverSLS_NM(-NList34[idx], MFYList34[idx])
            print("From analytical with N and M ...")

        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (extra_analytical[0], extra_analytical[1], extra_analytical[2])
        )
        error1 = abs((-extra[5] - extra_analytical[0]) / extra_analytical[0])
        # in code_aster compression is positive
        # in python  compression is negative
        error2 = abs((-extra[6] - extra_analytical[1]) / extra_analytical[1])
        error3 = abs((extra[7] - extra_analytical[2]) / extra_analytical[2])
        error_from_analitycal = max(error1, error2, error3)
        print("Error from analytical : %1.3E" % error_from_analitycal)
    else:
        print("Elastic algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Analytical algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Error from analytical : N.D.")

    error = (areaSteel34[idx] - Area) / (Area)
    if not (areaSteel34[idx] == -1000000.0):
        if abs(error) > 1e-02:
            idxerror.append(idx)
            print("INDICE ERRORE %1.0i" % idx)
            print(
                "ERRORE ECCESSIVO *******************************************************************"
            )
            raise Exception("ERRORE ECCESSIVO")
        errors.append(abs(error))
    else:
        errors_skipped.append(abs(error))
    print("Error: %1.3e" % error)
    print("===============")
    columnForCSV.append(
        [
            columnCSVName,
            "B450C",
            "C32/40",
            columnCSVPointN,
            columnCSVPointM,
            columnCSVArea,
        ]
    )

tableFerr56 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbferr56", "#PYTHON_END_tbferr56"
)
tableFerr56.parse()
ATOT = tableFerr56.fetchColumnWithName("ATOT")
ATOT5 = tableFerr56.fetchColumnWithName("ATOT", 5)
ATOT6 = tableFerr56.fetchColumnWithName("ATOT", 6)

tableEfge56 = au.AsterTableReader(
    fileName, "#PYTHON_START_tbefge56", "#PYTHON_END_tbefge56"
)
tableEfge56.parse()
N = tableEfge56.fetchColumnWithName("N")
MFY = tableEfge56.fetchColumnWithName("MFY")
MFZ = tableEfge56.fetchColumnWithName("MFZ")

MAILLE56 = tableFerr56.fetchColumnWithName("MAILLE")

areaSteel56 = []
# for idx, v in enumerate(ATOT):
#    areaSteel56.append(ATOT[idx]*1E+06)

MFYList56 = []
NList56 = []
B56 = []
H56 = []

for idx, _v in enumerate(MFY[: int(len(MFY) / 2)]):
    MFYi = ((MFY[idx * 2] + MFY[idx * 2 + 1]) / 2.0) * 1e03
    MFZi = ((MFZ[idx * 2] + MFZ[idx * 2 + 1]) / 2.0) * 1e03
    if abs(MFYi) > abs(MFZi) and abs(MFZi) / abs(MFYi) < 0.05:
        MFYList56.append(MFYi)
        B56.append(500.0)
        H56.append(300.0)
        NList56.append(-(N[idx * 2] + N[idx * 2 + 1]) / 2)
        areaSteel56.append(ATOT6[idx - (len(ATOT6))] * 1e06)
    elif abs(MFZi) > abs(MFYi) and abs(MFYi) / abs(MFZi) < 0.05:
        MFYList56.append(MFZi)
        B56.append(300.0)
        H56.append(500.0)
        NList56.append(-(N[idx * 2] + N[idx * 2 + 1]) / 2)
        areaSteel56.append(ATOT5[idx] * 1e06)

for idx, _v in enumerate(MFYList56):
    strs = f"{B56[idx]:1.0f}x{H56[idx]:2.0f}"

    columnCSVName = "section_03-" + strs
    columnCSVPointN = str("%1.2f" % (NList56[idx] * 1e-03))
    columnCSVPointM = str("%1.2f" % (MFYList56[idx] * 1e-06))

    strCase = (
        columnCSVName
        + "- mesh: "
        + MAILLE56[idx]
        + " - N="
        + str("%1.2f" % (NList56[idx] * 1e-03))
        + " [KN] - "
        + "M="
        + str("%1.2f" % (MFYList56[idx] * 1e-06))
        + " [KN*m]"
    )
    Area, extra, section = est.RCTemplRectEC2DesignNM(
        name=strCase,
        elementType="column",
        steel="B450C",
        concrete="C32/40",
        alphacc=1.00,
        sectionH=H56[idx],
        sectionW=B56[idx],
        topRecover=40.0,
        botRecover=40.0,
        N=NList56[idx],
        M=MFYList56[idx],
        toll=1e-5,
        toll_bisec=1e-5,
        maxIt=100,
        plot=plottingOnScreen,
        alphaTopBot=1,
        factorStart=0,
        factorEnd=1,
        points=100,
        logLevel=0,
        markersOnDomain=markers,
        savingSingleDomains=False,
        sigmacMax=0.65 * 32,
        sigmasMax=0.80 * 450,
        homogeneization=15.0,
        SLS=True,
        outSection=True,
    )
    columnCSVArea = "%1.3f" % Area
    print("===============")
    print("Case Test string:" + strCase)
    print("Area with bisection or secants: %1.3f" % Area)
    print("Area with FERRAILLAGE 1D: %1.3f" % areaSteel56[idx])

    if extra is not None:
        print("From elastic alghoritm ...")
        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (-extra[5], -extra[6], extra[7])
        )
        section.assignSteelAreaWithAlphaWeight(Area)
        if abs(NList56[idx]) < 0.0000005:
            # print(section)
            extra_analytical = section.solverSLS_M(MFYList56[idx])
            print("From analytical with only M ...")
        else:
            extra_analytical = section.solverSLS_NM(-NList56[idx], MFYList56[idx])
            print("From analytical with N and M ...")

        print(
            "sigma_c = %1.3E - sigma_s = %2.3E - xi = %3.3E "
            % (extra_analytical[0], extra_analytical[1], extra_analytical[2])
        )
        error1 = abs((-extra[5] - extra_analytical[0]) / extra_analytical[0])
        # in code_aster compression is positive
        # in python  compression is negative
        error2 = abs((-extra[6] - extra_analytical[1]) / extra_analytical[1])
        error3 = abs((extra[7] - extra_analytical[2]) / extra_analytical[2])
        error_from_analitycal = max(error1, error2, error3)
        print("Error from analytical : %1.3E" % error_from_analitycal)
    else:
        print("Elastic algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Analytical algorithm not defined ...")
        print("sigma_c = N.D. - sigma_s = N.D. - xi = N.D. ")
        print("Error from analytical : N.D.")

    if not (areaSteel56[idx] == -1000000.0):
        error = (areaSteel56[idx] - Area) / (Area)
        errors.append(abs(error))
    else:
        errors_skipped.append(abs(error))
    print("Error: %1.3e" % error)
    print("===============")
    columnForCSV.append(
        [
            columnCSVName,
            "B450C",
            "C32/40",
            columnCSVPointN,
            columnCSVPointM,
            columnCSVArea,
        ]
    )

print("===============")
print("   Nb tests: %1.0i" % len(errors))
print(" Nb skipped: %1.0i" % len(errors_skipped))
print("  Max error: %1.3e" % max(errors))
print("  Min error: %1.3e" % min(errors))
print("===============")

end = time.time()
print(end - start)

with open("for_latex_results_sls.csv", mode="w") as csv_file:
    csv_writer = csv.writer(
        csv_file, delimiter=",", quotechar='"', quoting=csv.QUOTE_MINIMAL
    )
    for r in columnForCSV:
        csv_writer.writerow(r)



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from enum import Enum
from pathlib import Path
from typing import Union, Any

from pydantic import BaseModel


# Example of use for solver is swowed below:
# 1. Build a solver:
#           solver = RcSecRectSolver()
# 2. Add a model input for calculations
#           solver.setModelInput(iData)
# 3. Setting a job path for artifacts or similar
#           solver.setJobPath(str(job_path))
# 4. Running a specific task:
#           solver.run(SolverOptions.STATIC)
# 5.
# oData = solver.getModelOutput()
#
class Solver:
    def __init__(self, modelInput: Union[BaseModel, None] = None):
        self.__modelInput: Union[BaseModel, None] = modelInput
        self.__modelLogs: Union[BaseModel, None] = None
        self.__modelOutput: Union[BaseModel, None] = None
        self.__modelResources: Union[BaseModel, None] = None
        self.__jobPath: str | Path = ""
        self.__modelInputFile: str = ""
        self.__modelOutputFile: str = ""
        self.__reportName: str = "report"
        self.__solverName: str = "Solver"

    def _setSolverName(self, name):
        self.__solverName = name

    def solverName(self) -> str:
        return self.__solverName

    def outPath(self) -> Path:
        return Path(self.__jobPath) / Path(self.__solverName)

    @property
    def reportName(self):
        return self.__reportName

    @reportName.setter
    def reportName(self, value):
        self.__reportName = value

    @property
    def modelOutputFile(self):
        return self.__modelOutputFile

    @modelOutputFile.setter
    def modelOutputFile(self, value):
        self.__modelOutputFile = value

    @property
    def modelInputFile(self):
        return self.__modelInputFile

    @modelInputFile.setter
    def modelInputFile(self, value):
        self.__modelInputFile = value

    def _setModelOutput(self, model: BaseModel) -> bool:
        self.__modelOutput = model
        return True

    def _setModelLogs(self, model: BaseModel) -> bool:
        self.__modelLogs = model
        return True

    def setModelInput(self, model: BaseModel) -> bool:
        ext = self._buildSolverFromModelInput(model)
        if ext:
            self.__modelInput = model
        return ext

    def _setModelResources(self, model: BaseModel) -> bool:
        self.__modelResources = model
        return True

    def setJobPath(self, path: str | Path) -> bool:
        self.__jobPath = path
        return True

    def getJobPath(self):
        return self.__jobPath

    def getModelOutput(self) -> Union[BaseModel, None]:
        return self.__modelOutput

    def getModelLogs(self) -> Union[BaseModel, None]:
        return self.__modelLogs

    def getModelInput(self):
        return self.__modelInput

    def getModelResources(self):
        return self.__modelResources

    # Pure Virtual Methods
    #
    def run(self, opt: Union[Enum, None] = None, **kwargs: Any) -> bool:
        raise NotImplementedError("Need to be implemented")

    def buildReport(self, opt: Union[Enum, None] = None, **kwargs: Any) -> bool:
        raise NotImplementedError("Need to be implemented")

    def buildResources(self, opt: Union[Enum, None] = None, **kwargs: Any) -> bool:
        raise NotImplementedError("Need to be implemented")

    def exportModelInput(self) -> BaseModel:
        return self._buildModelInputFromSolver()

    def _buildModelInputFromSolver(self) -> BaseModel:
        raise NotImplementedError("Need to be implemented")

    def _buildSolverFromModelInput(self, model: BaseModel) -> bool:
        raise NotImplementedError("Need to be implemented")



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

"""
Tests for EXAGeometry.shapes module

@author: lpaone
"""
import pytest

from pycivil.EXAGeometry.geometry import (
    Edge3d,
    Node2d,
    Node3d,
    Point2d,
    Point3d,
    Polyline2d,
)
from pycivil.EXAGeometry.shapes import Frame, ShapeCircle, ShapePoly


def test_Frame_with_ShapePoly() -> None:
    nodesLst = [
        Node2d(0.0, 0.0, 1),
        Node2d(300.0, 0.0, 2),
        Node2d(300.0, 500.0, 3),
        Node2d(0.0, 500.0, 4),
    ]
    poly = Polyline2d(nodesLst)
    shape = ShapePoly(poly)

    edge1 = Edge3d(Node3d(1, 1, 1, 1), Node3d(2, 3, 3, 3))
    frame1 = Frame(edge1)

    frame1.setShape(shape)

    # Build a normal frame
    n1 = Node3d(1.0, 1.0, 1.0, 1)
    n2 = Node3d(6.0, 6.0, 6.0, 2)

    axis = Edge3d(n1, n2)

    frame_typical = Frame(axis)

    frame_typical.setReference(Point3d(1.0, 1.0, 0.0))
    frame_typical.setReference(1.0, 1.0, 0.0)

    # Polygonar
    secPoly = ShapePoly(0.0, 0.0, 1, 300.0, 0.0, 2, 300.0, 500.0, 3, 0.0, 500.0, 4)

    frame_typical.setShape(secPoly)

    secPoly.translate(Point2d(150.0, 250.0), Point2d(0.0, 0.0))

    axis = Edge3d(1.0, 1.0, 1.0, 1, 6.0, 6.0, 6.0, 2)

    frame_typical = Frame(100.0, 100.0, 100.0, 1, 600.0, 600.0, 600.0, 2)
    frame_typical.setReference(100.0, 100.0, 0.0)
    frame_typical.setShape(secPoly)

    frame_typical2 = Frame(15.0, 25.0, 35.0, 1, 60.0, 70.0, 90.0, 2)
    frame_typical2.setReference(15.0, 25.0, 10.0)
    frame_typical2.setShape(secPoly)


def test_Frame_properties() -> None:
    edge1 = Edge3d(Node3d(1, 1, 1, 1), Node3d(2, 3, 3, 3))
    edge2 = Edge3d(Node3d(1, 3, 3, 3), Node3d(2, 3, 3, 0))

    Frame(edge1)
    Frame(edge2)

    with pytest.raises(TypeError):
        Frame(1)

    with pytest.raises(TypeError):
        Frame(1, 1)


def test_ShapePoly_properties() -> None:
    nodesLst = [
        Node2d(0.0, 0.0, 1),
        Node2d(300.0, 0.0, 2),
        Node2d(300.0, 500.0, 3),
        Node2d(0.0, 500.0, 4),
    ]
    poly = Polyline2d(nodesLst)

    ShapePoly(poly)


def test_ShapeCircle_properties() -> None:
    # First constructor
    shape1 = ShapeCircle(20, 100, 200)
    assert shape1.getRadius() == 20
    assert shape1.getShapePoint("O").x == 100
    assert shape1.getShapePoint("O").y == 200

    # Second constructor
    shape2 = ShapeCircle(20, center=Point2d(101, 201))
    assert shape2.getRadius() == 20
    assert shape2.getShapePoint("O").x == 101
    assert shape2.getShapePoint("O").y == 201



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pathlib import Path

import pycivil.EXAStructural.lawcodes.codeNTC2018 as Code
import pytest
from pycivil.EXAUtils.tables import Ex, Table


def test_table_codeNTC2018(tmp_path: Path):
    table = Code.WindActionsZones.tab01

    tab = Table(table)
    descr_1 = (
        "Valle d'Aosta, Piemonte, Lombardia,"
        "Trentino Alto Adige, Veneto, "
        "Friuli Venezia Giulia "
        "(con l'eccezione della provincia di Trieste)"
    )
    assert tab.valueAt(1) == (1, "Zona 1", descr_1, 25, 1000, 0.4)
    tab.setChoose(3)
    assert tab.choose() == 3
    pytest.raises(Ex, tab.setChoose, 0)

    print(tab)



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import math
from enum import Enum
from typing import Union

from pycivil.EXAGeometry.geometry import (
    Edge3d,
    Node2d,
    Node3d,
    Point2d,
    Point3d,
    Polyline2d,
    Vector3d,
)


class Frame:
    def __init__(self, *args):
        if len(args) == 0:
            raise TypeError("Only one argument !!!")
        elif len(args) == 1:
            if isinstance(args[0], Edge3d):
                self.__axis = args[0]
                self.__id = -1
                self.__reference = Point3d()
                self.__shape = None
            else:
                raise TypeError("Arg must be [Edge3D] !!!")
        elif len(args) == 8:
            for i in range(2):
                if not isinstance(args[0 + 4 * i], float):
                    raise TypeError("Arg n. %e must be float !!!" % (0 + i))
                if not isinstance(args[1 + 4 * i], float):
                    raise TypeError("Arg n. %e must be float !!!" % (1 + i))
                if not isinstance(args[2 + 4 * i], float):
                    raise TypeError("Arg n. %e must be float !!!" % (2 + i))
                if not isinstance(args[3 + 4 * i], int):
                    raise TypeError("Arg n. %e must be int !!!" % (3 + i))
            self.__axis = Edge3d(
                Node3d(args[0], args[1], args[2], args[3]),
                Node3d(args[4], args[5], args[6], args[7]),
            )
            self.__id = -1
            self.__reference = Point3d()
            self.__shape = None
        else:
            raise TypeError("Wrong arguments !!!")

    def __str__(self):
        dispstr = "Frame Object: \n"
        dispstr = dispstr + "------------- \n"
        dispstr = dispstr + "id: " + str(self.__id) + "\n"
        dispstr = dispstr + "axis--> \n"
        dispstr = dispstr + str(self.__axis)
        dispstr = dispstr + "reference--> \n"
        dispstr = dispstr + str(self.__reference) + "\n"
        if self.__shape is not None:
            dispstr = dispstr + "Shape--> \n"
            dispstr = dispstr + str(self.__shape) + "\n"
        return dispstr

    # Fix y local axis
    def setReference(self, *args):
        if len(args) == 1:
            if isinstance(args[0], Point3d):
                self.__reference = args[0]
            else:
                raise Exception("Arg must be [Point3D] !!!")
        elif len(args) == 3:
            if not isinstance(args[0], float):
                raise Exception("Arg nb 1 must be float !!!")
            if not isinstance(args[1], float):
                raise Exception("Arg nb 2 must be float !!!")
            if not isinstance(args[2], float):
                raise Exception("Arg nb 1 must be float !!!")
            self.__reference = Point3d(args[0], args[1], args[2])
        else:
            raise Exception("Wrong arguments !!!")

    def getXLocalAxis(self):
        return Vector3d(self.__axis.nodeI(), self.__axis.nodeJ()).normalize()

    def getZLocalAxis(self):
        xAxis = self.getXLocalAxis()
        yAxisDir = Vector3d(self.__axis.nodeI(), self.__reference)
        if xAxis.cross(yAxisDir).norm() == 0:
            raise Exception("Null cross product: referenze point bad defined !!!")
        return xAxis.cross(yAxisDir).normalize()

    def getYLocalAxis(self):
        return self.getZLocalAxis().cross(self.getXLocalAxis())

    # Shape
    def setShape(self, *args):
        if len(args) == 1:
            if isinstance(args[0], Shape):
                self.__shape = args[0]
            else:
                raise Exception("Arg must be [Shape] !!!")
        else:
            raise Exception("Wrong arguments !!!")

    def nodeI(self):
        return self.__axis.nodeI()

    def nodeJ(self):
        return self.__axis.nodeJ()

    def lenght(self):
        return self.__axis.lenght()


class ShapesEnum(int, Enum):
    SHAPE_NONE = -1
    SHAPE_POLY = 1
    SHAPE_RECT = 101
    SHAPE_CIRC = 102
    SHAPE_AREA = 103


class Shape:
    def __init__(
        self,
        ids: int = -1,
        typ: ShapesEnum = ShapesEnum.SHAPE_NONE,
        originX: float = 0.0,
        originY: float = 0.0,
    ):
        self.__ids = ids
        self.__type = typ
        self.__o = Point2d(originX, originY)

    def getIds(self):
        return self.__ids

    def getType(self) -> ShapesEnum:
        return self.__type

    def getDesc(self):
        return "generic"

    def setId(self, ids):
        self.__ids = ids

    def setOrigin(self, point: Union[Point2d, None]) -> None:
        if point is None:
            point = Point2d(0.0)

        self.__o.x = point.x
        self.__o.y = point.y

    def getOrigin(self):
        return self.__o

    def getArea(self) -> float:
        raise NotImplementedError("getArea() not implemented for subclass !!!")

    def getDiameter(self):
        return 2 * math.sqrt(self.getArea() / math.pi)

    def getShapePoint(self, idstr: str) -> Point2d:
        if isinstance(idstr, str):
            if idstr == "O":
                return self.getOrigin()
            elif idstr == "G":
                return self.getOrigin()
            else:
                raise ValueError(f'Point str {idstr} undefined !!!')

        return Point2d()

    def translate(
        self, pStart: Union[Point2d, None], pEnd: Union[Point2d, None]
    ) -> None:
        if pStart is None:
            pStart = Point2d()
        if pEnd is None:
            pEnd = Point2d()

        assert isinstance(pStart.x, (float, int))
        assert isinstance(pStart.y, (float, int))
        assert isinstance(pEnd.x, (float, int))
        assert isinstance(pEnd.y, (float, int))
        self.__o.translate(pEnd.x - pStart.x, pEnd.y - pStart.y)

    def vertexNb(self):
        return 0

    def vertexAt(self, i):
        raise Exception("Vertex for [Shape] objects not defined !!!")

    def vertexMaxInY(self):
        if self.vertexNb() == 0:
            raise Exception("Cannot find in 0 vertices number max in Y !!!")

        val = []
        vertex = []
        for i in range(self.vertexNb()):
            val.append(self.vertexAt(i).y)
            vertex.append(self.vertexAt(i))

        return max(val), vertex[val.index(max(val))]

    def vertexMaxInX(self):
        if self.vertexNb() == 0:
            raise Exception("Cannot find in 0 vertices number max in X !!!")

        val = []
        vertex = []
        for i in range(self.vertexNb()):
            val.append(self.vertexAt(i).x)
            vertex.append(self.vertexAt(i))

        return max(val), vertex[val.index(max(val))]

    def vertexMinInY(self):
        if self.vertexNb() == 0:
            raise Exception("Cannot find in 0 vertices number min in Y !!!")

        val = []
        vertex = []
        for i in range(self.vertexNb()):
            val.append(self.vertexAt(i).y)
            vertex.append(self.vertexAt(i))

        return min(val), vertex[val.index(min(val))]

    def vertexMinInX(self):
        if self.vertexNb() == 0:
            raise Exception("Cannot find in 0 vertices number min in X !!!")

        val = []
        vertex = []
        for i in range(self.vertexNb()):
            val.append(self.vertexAt(i).x)
            vertex.append(self.vertexAt(i))
        return min(val), vertex[val.index(min(val))]

    def __str__(self) -> str:
        dispstr = "Shape Object: \n"
        dispstr = dispstr + "--------------- \n"
        dispstr = dispstr + "  id: " + str(self.__ids) + "\n"
        dispstr = dispstr + "type: " + str(self.__type) + " " + self.getDesc() + "\n"
        return dispstr


class ShapePoly(Shape):
    def __init__(self, *args):
        if len(args) == 0:
            raise Exception("Only one or >= 9 argument !!!")
        elif len(args) == 1:
            Shape.__init__(self, -1, ShapesEnum.SHAPE_POLY, 0, 0)
            if isinstance(args[0], Polyline2d):
                self.__polyline = args[0]
            else:
                raise Exception("Arg unknown !!!")
        elif len(args) >= 9:
            Shape.__init__(self, -1, ShapesEnum.SHAPE_POLY, 0, 0)
            # lst = []
            if (len(args) / 3.0 - int(len(args) / 3.0)) != 0:
                raise Exception("Args must be multiple of 3 !!!")
            idx = int(len(args) / 3.0)
            lst = []
            for i in range(idx):
                if not isinstance(args[0 + i * 3], float):
                    raise Exception("Args 1*n must be only float !!!")
                if not isinstance(args[1 + i * 3], float):
                    raise Exception("Args 2*n must be only float !!!")
                if not isinstance(args[2 + i * 3], int):
                    raise Exception("Args 3*n must be only int !!!")
                lst.append(Node2d(args[0 + i * 3], args[1 + i * 3], args[2 + i * 3]))
            self.__polyline = Polyline2d(lst)
        else:
            raise Exception("Wrong arguments !!!")

    def getShapePoint(self, idstr: str) -> Point2d:
        if isinstance(idstr, str):
            if idstr == "O":
                return self.getOrigin()
            elif idstr == "G":
                raise Exception("Not defined barycenter for Poliline2D!!!")
            else:
                raise Exception("Point str undefined !!!")
        else:
            raise ValueError("idstr must be str !!!")

    def getDesc(self):
        return "polygonar"

    def __str__(self) -> str:
        return (
            super().__str__()
            + "\n"
            + "Data embedded-->\n"
            + self.__polyline.__str__()
            + "\n"
        )

    def translate(self, pStart: Union[Point2d, None], pEnd: Union[Point2d, None]) -> None:

        super().translate(pStart, pEnd)
        self.__polyline.translate(pStart, pEnd)

    def getPolyline2D(self):
        return self.__polyline


class ShapeRect(Shape):
    """Class that instantiate a rectangular shape with width and height

    Class that instantiate a rectangular shape with width and height.
    The origin point from default is in x = 0 y = 0.
    Special points are TL (top left), TR (top right), BL (bottom left),
    BR (bottom right), MB (medium bottom), MT (medium top), O (origin),
    G (geometrical barycenter).

    """

    def __init__(
        self,
        width: float = 0.0,
        height: float = 0.0,
        ids: int = -1,
        xo: float = 0.0,
        yo: float = 0.0,
    ):
        super().__init__(ids, ShapesEnum.SHAPE_RECT, xo, yo)
        self.__w = width
        self.__h = height
        self.__vertexFromMDimensions()

    def __vertexFromMDimensions(self):
        o = self.getOrigin()
        assert type(o.x) is float
        assert type(o.y) is float
        self.__TL = Point2d(o.x - self.__w / 2.0, o.y + self.__h / 2.0)
        self.__TR = Point2d(o.x + self.__w / 2.0, o.y + self.__h / 2.0)
        self.__BL = Point2d(o.x - self.__w / 2.0, o.y - self.__h / 2.0)
        self.__BR = Point2d(o.x + self.__w / 2.0, o.y - self.__h / 2.0)

    def w(self):
        return self.__w

    def h(self):
        return self.__h

    def setDimH(self, h):
        self.__h = h
        self.__vertexFromMDimensions()

    def setDimW(self, w):
        self.__w = w
        self.__vertexFromMDimensions()

    def getArea(self) -> float:
        return self.__w * self.__h

    def getDesc(self):
        return "rectangular"

    def getShapePoint(
        self, idstr: str
    ) -> Point2d:
        """
        Retrieve special named points for shapes

        Args:
            idstr (): idstr must be one of ["O", "G", "TL", "TR", "BL", "BR", "MB", "MT"]

        Returns:

        """
        if isinstance(idstr, str):
            if idstr == "O":
                return self.getOrigin()
            elif idstr == "G":
                return (self.__TL + self.__BR) * 0.5
            elif idstr == "TL":
                return self.__TL
            elif idstr == "TR":
                return self.__TR
            elif idstr == "BL":
                return self.__BL
            elif idstr == "BR":
                return self.__BR
            elif idstr == "MB":
                return (self.__BL + self.__BR) * 0.5
            elif idstr == "MT":
                return (self.__TL + self.__TR) * 0.5
            else:
                raise ValueError("Point str undefined !!!")
        else:
            raise ValueError("idstr must be a str type !!!")

    def translate(
        self, pStart: Union[Point2d, None], pEnd: Union[Point2d, None]
    ) -> None:
        if pStart is None:
            pStart = Point2d()
        if pEnd is None:
            pEnd = Point2d()

        super().translate(pStart, pEnd)

    def vertexNb(self):
        return 4

    def vertexAt(self, i):
        if i == 0:
            return self.getShapePoint("BL")
        elif i == 1:
            return self.getShapePoint("BR")
        elif i == 2:
            return self.getShapePoint("TL")
        elif i == 3:
            return self.getShapePoint("TR")
        else:
            raise Exception("i out of the bound !!!")

    def __str__(self) -> str:
        dispstr = super().__str__()
        dispstr = dispstr + "Data embedded-->\n"
        dispstr = dispstr + "b = " + str(self.__w) + " h = " + str(self.__h) + "\n"
        dispstr = dispstr + "Origin in " + self.getOrigin().__str__() + "\n"
        return dispstr


class ShapeCircle(Shape):
    def __init__(
        self,
        radius: float = 0.0,
        posX: float = 0.0,
        posY: float = 0.0,
        ids: int = -1,
        center: Point2d = Point2d(make_null=True),
    ):
        self.__r: float = radius
        if not center.isNull():
            posX = center.x
            posY = center.y

        super().__init__(ids, ShapesEnum.SHAPE_CIRC, posX, posY)

    def getRadius(self) -> float:
        return self.__r

    def getDesc(self) -> str:
        return "circle"

    def getShapePoint(self, idstr: str) -> Point2d:
        """
        Retrieve special named points for shapes

        Args:
            idstr (): idstr must be one of ["O", "G", "C"]

        Returns:

        """
        if isinstance(idstr, str):
            if idstr == "O" or idstr == "G" or idstr == "C":
                return Point2d(self.getOrigin().x, self.getOrigin().y)
            else:
                raise ValueError(f'Point str {idstr} undefined !!!')
        else:
            raise ValueError("idstr arg must be a str !!!")

    def center(self) -> Point2d:
        return Point2d(self.getOrigin().x, self.getOrigin().y)

    def translate(
        self, pStart: Union[Point2d, None], pEnd: Union[Point2d, None]
    ) -> None:

        if pStart is None:
            pStart = Point2d()
        if pEnd is None:
            pEnd = Point2d()

        return super().translate(pStart, pEnd)

    def __str__(self) -> str:
        dispstr = super().__str__()
        dispstr = dispstr + "Data embedded-->\n"
        dispstr = dispstr + "r = " + str(self.__r) + "\n"
        dispstr = dispstr + "Origin in " + self.getOrigin().__str__() + "\n"
        return dispstr


class ShapeArea(Shape):
    def __init__(
        self, area: float = 0.0, ids: int = -1, xo: float = 0.0, yo: float = 0.0
    ):
        Shape.__init__(self, ids, ShapesEnum.SHAPE_AREA, xo, yo)
        self.__area = area

    def setArea(self, a):
        self.__area = a

    def getArea(self) -> float:
        return self.__area

    def getDesc(self):
        return "area"

    def getShapePoint(self, idstr: str) -> Point2d:

        if isinstance(idstr, str):
            if idstr == "O":
                return self.getOrigin()
            elif idstr == "G":
                return self.getOrigin()
            else:
                raise Exception("Point str undefined !!!")

        return Point2d()

    def __str__(self) -> str:
        dispstr = super().__str__()
        dispstr = dispstr + "Data embedded-->\n"
        dispstr = dispstr + "area = " + str(self.__area) + "\n"
        dispstr = dispstr + "Origin in " + self.getOrigin().__str__() + "\n"
        return dispstr



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import math
from pathlib import Path

import pytest
from pycivil.EXAStructural.codes import Code
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pycivil.EXAStructural.sections import SectionStates
from pycivil.EXAStructural.templateRCRect import RCTemplRectEC2


# ==============================================================================
# BY-HAND TEST CASE #5 - Manuscript 22/1/26
# ==============================================================================
# Section geometry:
#   - Width B = 1000 mm
#   - Height H = 500 mm
#
# Materials:
#   - Concrete C32/40:
#       f_ck = 32 MPa
#       f_ctm = 0.30 * f_ck^(2/3) = 3.02 MPa
#       f_ctm_crack = f_ctm / 1.2 = 2.52 MPa
#       E_cm = 22000 * (f_ck/10)^0.3 = 33545 MPa
#   - Steel:
#       E_s = 200'000 MPa
#       n = E_s / E_cm ≈ 6
#
# Reinforcement:
#   - Top: 5∅24 -> A_s = 452.2 mm² at d = 492.9 mm from bottom (cover 7.4 mm)
#   - Bottom: 5∅18 -> A_s = 254.3 mm² at d = 7.1 mm from bottom (cover 7.1 mm)
#
# UNCRACKED SECTION (Sezione non fessurata):
#   Neutral axis calculation:
#     x_f = [A_s1*d1*n + A_s2*d2*n + B*H*(H/2)] / [A_s1*n + A_s2*n + B*H]
#     x_f = [452.2*492.9*5.6 + 254.3*7.1*5.6 + 1000*500*250] /
#           [452.2*5.6 + 254.3*5.6 + 1000*500]
#     x_f = [6.687e6 + 54.16e3 + 125e6] / [13.57e3 + 7.63e3 + 500e3]
#     x_f = 252.76 mm (from bottom)
#
#   Moment of inertia (uncracked):
#     J_cls = (1/12)*B*H³ + B*H*(H/2 - x_f)²
#           = (1/12)*1000*500³ + 1000*500*(2.76)² = 10.42e9 + 3.81e6 ≈ 10.42e9 mm⁴
#     J_Acc = A_s1*(d1-x_f)²*n + A_s2*(x_f-d2)²*n
#           = 452.2*233.8²*5.6 + 254.3*245.66²*5.6 = 780.1e6 + 460.4e6 = 1.24e9 mm⁴
#     J_i^I = J_cls + J_Acc = 11.66e9 mm⁴ = 1'166'000 cm⁴
#
#   First cracking moment:
#     M_cr = (f_ctm * J_i^I) / y_cr = (2.52 * 11.66e9) / 252.76 = 116.25 kN.m
#
# CRACKED SECTION (Sezione fessurata):
#   Neutral axis (from compression edge):
#     Solving quadratic equation for x:
#     x = [-n*(A_s1+A_s2) + sqrt((n*(A_s1+A_s2))² + 2*B*n*(A_s1*d1+A_s2*d2))] / B
#     x ≈ 69.15 mm (from top/compression edge)
#
#   Moment of inertia (cracked):
#     J_cls = (1/12)*B*x³ + B*x*(x/2)² (only compressed concrete)
#     J_Acc = A_s1*(d1'-x)²*n + A_s2*(d2'-x)²*n (d' from compression edge)
#     J_i^II = 1.42e9 mm⁴ = 139'225 cm⁴
#
#   Stiffness ratio:
#     J_i^I / J_i^II ≈ 8.21
# ==============================================================================


# --------------------
# BY-HAND TEST CASE #3
# --------------------
def test_symmetrical_uncracked_NM_001(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2018")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C25/30")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(600)
    rcSection.setDimW(300)

    # Longitudinal rebars
    lineMTRebars = rcSection.addSteelArea("LINE-MT", dist=50, d=20, nb=4, sd=40)
    lineMBRebars = rcSection.addSteelArea("LINE-MB", dist=50, d=20, nb=4, sd=40)
    assert isinstance(lineMTRebars, list)
    assert isinstance(lineMBRebars, list)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea()) == 180000
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 2513
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 217699
    assert pytest.approx(rcSection.calProp_Ihx(), rel=1e-3) == 7.755e09
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y]
    ) == [0.0, 0.0]

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [0, 0, None]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == 0
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == 0

    sigmac, sigmas, xi = rcSection.solverSLS_NM(1000 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [4.593, 68.895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == 4.593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == 68.895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-1000 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-4.593, -68.895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == -4.593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == -68.895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-100 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [
        -0.4593,
        -68.895 / 10,
        math.inf,
    ]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == -0.4593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == -6.8895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(+100 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [+0.4593, +6.8895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == +0.4593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == +6.8895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, 150.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-5.803, 72.53, 300]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == +5.803
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == -5.803
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineMBRebars:
            stressExpected = +72.53
        if idx in lineMTRebars:
            stressExpected = -72.53
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-3) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, -150.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-5.803, +72.53, 300]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -5.803
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == +5.803
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0.0
        if idx in lineMBRebars:
            stressExpected = -72.53
        if idx in lineMTRebars:
            stressExpected = +72.53
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-3) == stressExpected

    # Neutral axis is measured from bottom to top of section
    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        -1000.0 * KN, +150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-10.396, +3.635, 537.45]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == +1.210
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -10.396
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0.0
        if idx in lineMBRebars:
            stressExpected = +3.635
        if idx in lineMTRebars:
            stressExpected = -141.425
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        -1000.0 * KN, -150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-10.396, +3.635, 62.65]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == -10.396
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == +1.210
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0.0
        if idx in lineMBRebars:
            stressExpected = -141.425
        if idx in lineMTRebars:
            stressExpected = +3.635
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        +1000.0 * KN, +150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-1.210, +141.425, 62.55]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == 10.396
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -1.210
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0.0
        if idx in lineMBRebars:
            stressExpected = +141.425
        if idx in lineMTRebars:
            stressExpected = -3.635
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        +1000.0 * KN, -150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-1.210, +141.425, 537.45]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == -1.210
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == 10.396
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0.0
        if idx in lineMBRebars:
            stressExpected = -3.635
        if idx in lineMTRebars:
            stressExpected = +141.425
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.2
# ----------------------
def test_uncracked_NM_002(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)
    assert isinstance(lineRebars01, list)
    assert isinstance(lineRebars02, list)
    assert isinstance(lineRebars03, list)
    assert isinstance(lineRebars04, list)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea(), rel=1e-6) == 960.00e03
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 18.09e03
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 1.23135e06
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y],
        rel=1e-2,
        abs=1e-3,
    ) == [0.0, -220.89]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y], rel=1e-2, abs=1
    ) == [0.0, -49.39]
    assert (
        pytest.approx(rcSection.calProp_Ihx(barycenter=True), rel=1e-3, abs=1e-3)
        == 176.24e09
    )

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, 2400.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-8.84, +96.97, 649.39]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == +7.4988
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == -8.840

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineRebars01:
            stressExpected = -117.11
        if idx in lineRebars02:
            stressExpected = +74.91
        if idx in lineRebars03:
            stressExpected = +85.94
        if idx in lineRebars04:
            stressExpected = +96.97
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.2
# ----------------------
def test_uncracked_NM_003(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=130, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=184, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)
    assert isinstance(lineRebars01, list)
    assert isinstance(lineRebars02, list)
    assert isinstance(lineRebars03, list)
    assert isinstance(lineRebars04, list)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea(), rel=1e-6) == 960.00e03
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 18.09e03
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 1.23135e06
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y],
        rel=1e-2,
        abs=1e-3,
    ) == [0.0, +220.89]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y], rel=1e-2, abs=1
    ) == [0.0, +49.39]
    assert (
        pytest.approx(rcSection.calProp_Ihx(barycenter=True), rel=1e-3, abs=1e-3)
        == 176.24e09
    )

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, -2400.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-8.84, +96.97, 550.61]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -8.840
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == +7.4988

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineRebars04:
            stressExpected = -117.11
        if idx in lineRebars03:
            stressExpected = +74.91
        if idx in lineRebars02:
            stressExpected = +85.94
        if idx in lineRebars01:
            stressExpected = +96.97
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.5
# ----------------------
def test_cracked_stretched_NM_004(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)
    assert isinstance(lineRebars01, list)
    assert isinstance(lineRebars02, list)
    assert isinstance(lineRebars03, list)
    assert isinstance(lineRebars04, list)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(+3619 * KN, 0.0 * KNm, uncracked=False)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [0.0, +376.27, 1666.0]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == 0.0
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == 0.0

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineRebars01:
            stressExpected = 376.27
        if idx in lineRebars02:
            stressExpected = 153.86
        if idx in lineRebars03:
            stressExpected = 141.05
        if idx in lineRebars04:
            stressExpected = 128.27
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.6
# ----------------------
def test_cracked_compressed_NM_005(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)
    assert isinstance(lineRebars01, list)
    assert isinstance(lineRebars02, list)
    assert isinstance(lineRebars03, list)
    assert isinstance(lineRebars04, list)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-3619 * KN, 0.0 * KNm, uncracked=False)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-3.599, -36.92, 3557.496]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -2.385
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -3.599

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineRebars01:
            stressExpected = -52.84
        if idx in lineRebars02:
            stressExpected = -38.56
        if idx in lineRebars03:
            stressExpected = -37.74
        if idx in lineRebars04:
            stressExpected = -36.92
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.1
# ----------------------
def test_cracked_bending_NM_006(tmp_path: Path) -> None:
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)
    assert isinstance(lineRebars01, list)
    assert isinstance(lineRebars02, list)
    assert isinstance(lineRebars03, list)
    assert isinstance(lineRebars04, list)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, 2400.0 * KNm, uncracked=False)

    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-10.31, +207.10, 480.51]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == 0.0
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -10.31

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected: float = 0
        if idx in lineRebars01:
            stressExpected = -130.19
        if idx in lineRebars02:
            stressExpected = +172.34
        if idx in lineRebars03:
            stressExpected = +189.72
        if idx in lineRebars04:
            stressExpected = +207.10
        assert pytest.approx(i, rel=1e-2) == stressExpected


def section_manuscript_5(mirr: bool = False):
    code = Code("NTC2008")
    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")
    rcSection = RCTemplRectEC2(1, "Template RC Section - Manuscript #5")
    rcSection.setDimH(500)
    rcSection.setDimW(1000)
    lineRebarsTop = rcSection.addSteelArea("LINE-MT" if not mirr else "LINE-MB", dist=74, d=24, nb=5, sd=200)
    lineRebarsBot = rcSection.addSteelArea("LINE-MB" if not mirr else "LINE-MT", dist=71, d=18, nb=5, sd=200)
    assert isinstance(lineRebarsTop, list)
    assert isinstance(lineRebarsBot, list)
    return rcSection

# -----------------------------------------
# BY-HAND TEST CASE #5 - Manuscript 22/1/26
# -----------------------------------------
def test_manuscript_uncracked_cracked_NM_007(tmp_path: Path) -> None:
    """
    Test case from manuscript #5 dated 22/1/26.

    Section geometry:
        - Width B = 1000 mm
        - Height H = 500 mm

    Materials:
        - Concrete C32/40:
            f_ck = 32 MPa
            f_ctm = 0.30 * f_ck^(2/3) = 3.02 MPa
            f_ctm_crack = f_ctm/1.2 = 2.52
            E_cm = 22000 * (f_ck/10)^0.3 = 33545 MPa
        - Steel:
            E_s = 200'000 MPa
            n = E_s / E_cm ≈ 6

    Reinforcement:
        - Top: 5∅24 -> A_s = 5 * 452.4 = 2262 mm² at d = 492.9mm from bottom
          (single bar area: π*24²/4 = 452.4 mm²)
        - Bottom: 5∅18 -> A_s = 5 * 254.5 = 1272 mm² at d = 7.1mm from bottom
          (single bar area: π*18²/4 = 254.5 mm²)
        - Total steel area: 2262 + 1272 = 3534 mm²

    UNCRACKED SECTION (Sezione non fessurata):
        Neutral axis calculation:
            x_f = [A_s1*d1*n + A_s2*d2*n + B*H*(H/2)] / [A_s1*n + A_s2*n + B*H]
            x_f = [2262*492.9*6 + 1272*7.1*6 + 1000*500*250] /
                  [2262*6 + 1272*6 + 1000*500]
            x_f = 252.76 mm (from bottom)

        Moment of inertia (uncracked):
            J_i^I = 11.66e9 mm⁴ = 1'166'000 cm⁴

        First cracking moment:
            M_cr = (f_ctm * J_i^I) / y_cr = (2.52 * 11.66e9) / 252.76 = 116.25 kN.m

    CRACKED SECTION (Sezione fessurata):
        Neutral axis: x = 69.15 mm from compression edge
        Moment of inertia: J_i^II = 1.42e9 mm⁴ = 139'225 cm⁴
        Stiffness ratio: J_i^I / J_i^II ≈ 8.21
    """
    rcSection = section_manuscript_5()

    homo_coeff = 6.0
    rcSection.setHomogenization(homo_coeff)

    # Geometrical properties from manuscript
    # Total steel area: 5*π*24²/4 + 5*π*18²/4 = 2262 + 1272 = 3534 mm²
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-4) == 3534

    # Concrete area: B * H = 1000 * 500 = 500000 mm²
    assert pytest.approx(rcSection.calConcreteArea(), rel=1e-4) == 500000

    # yg is calculated starting from origin fixed at middle of rectangle

    yg = rcSection.calBarycenter().y
    assert pytest.approx(yg, rel=1e-3) == 1.96

    # Uncracked moment of inertia
    # Manuscript value (with n≈6): J_i^I = 11.66e9 mm⁴
    # Code uses actual modular ratio from materials, resulting in slightly higher value
    Icx = rcSection.calProp_Icx(yg=yg)
    assert pytest.approx(Icx, rel=1e-3) == 10.42e9

    Isx = rcSection.calProp_Isx(yg=yg)
    assert pytest.approx(Isx, rel=1e-3) == 110.12e6

    Ihx = rcSection.calProp_Ihx(barycenter=True)
    assert Ihx > 10e9  # Sanity check: should be greater than pure concrete section

    Mcr_pos, Mcr_neg = rcSection.calCriticalMoment()
    assert pytest.approx(110.83e6, rel=1e-3) == Mcr_pos
    assert -Mcr_neg > Mcr_pos

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    # Test uncracked section with pure bending at first cracking moment
    # M_cr = 116.25 kN.m (from manuscript with J_i^I = 11.66e9, y_cr = 252.76mm)
    # f_ctm = 3.02 MPa
    rcSection.solverSLS_NM(0.0 * KN, 110.83 * KNm, uncracked=True)

    # Concrete stress at bottom must be close to f_ctm value with Mcr as effort
    sigmac_BL = rcSection.getConcrStress()[0]
    assert pytest.approx(sigmac_BL, rel=1e-3) == 2.52

def test_manuscript_get_area_properties():
    # Build section
    rcSection = section_manuscript_5()
    homo_coeff = 6.0
    rcSection.setHomogenization(homo_coeff)

    # then for uncracked (default)
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties()
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(48.3, rel=1e-2) == ysg
    assert pytest.approx(1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(-980e+3, rel=1e-3) == Scx
    assert pytest.approx(163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # then for uncracked
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.UNCRACKED)
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(48.3, rel=1e-2) == ysg
    assert pytest.approx(1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(-980e+3, rel=1e-3) == Scx
    assert pytest.approx(163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # then for compressed
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.COMPRESSED)
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(48.3, rel=1e-2) == ysg
    assert pytest.approx(1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(-980e+3, rel=1e-3) == Scx
    assert pytest.approx(163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # If i didn't launch solverSLS_NM before for PARTIALIZED i can't have results
    assert rcSection.getAreaProperties(SectionStates.PARTIALIZED) is None


    # Launch withowt forces
    # solverSLS_NM classifies section properly lucly
    rcSection.solverSLS_NM(N=0.0e+3, M=150e+6, uncracked=False)
    assert pytest.approx(73.46, rel=1e-2) == rcSection.xi()

    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.PARTIALIZED)
    assert pytest.approx(73.46e+3, rel=1e-2) == Ac
    assert pytest.approx( 3.53e+3, rel=1e-2) == As
    assert pytest.approx(94.63e+3, rel=1e-2) == Ah
    assert pytest.approx(  213.27, rel=1e-2) == ycg
    assert pytest.approx(    48.3, rel=1e-2) == ysg
    assert pytest.approx(  176.67, rel=1e-2) == yhg
    assert pytest.approx(2.68e+6, rel=1e-1) == Scx
    assert pytest.approx(-453.1e+3, rel=1e-2) == Ssx
    assert pytest.approx(    0.0, abs=1e-6) == Shx
    assert pytest.approx(131.43e+6, rel=1e-1) == Jcx
    assert pytest.approx(1.60e+8, rel=1e-2) == Jsx
    assert pytest.approx(1.091e+9, rel=1e-2) == Jhx

    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.STRETCHED)
    assert pytest.approx(     0.0, rel=1e-16) == Ac
    assert pytest.approx( 3.53e+3, rel=1e-2)  == As
    assert pytest.approx( 3.53e+3, rel=1e-2)  == Ah
    assert pytest.approx(  0.0, rel=1e-2)     == ycg
    assert pytest.approx(    48.3, rel=1e-2) == ysg
    assert pytest.approx(    48.3, rel=1e-2) == yhg
    assert pytest.approx( 0.0, rel=1e-16) == Scx
    assert pytest.approx( 0.0, abs=1e-10) == Ssx
    assert pytest.approx( 0.0, abs=1e-10) == Shx
    assert pytest.approx( 0.0, rel=1e-16) == Jcx
    assert pytest.approx(102.46e+6, rel=1e-2) == Jsx
    assert pytest.approx(102.46e+6, rel=1e-2) == Jhx

def test_manuscript_get_area_properties_mirrored():
    rcSection = section_manuscript_5(mirr=True)
    homo_coeff = 6.0
    rcSection.setHomogenization(homo_coeff)

    Mcr_pos, Mcr_neg = rcSection.calCriticalMoment()
    assert pytest.approx(-110.83e6, rel=1e-3) == Mcr_neg
    assert Mcr_pos > -Mcr_neg

    # then for uncracked (default)
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties()
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(-48.3, rel=1e-2) == ysg
    assert pytest.approx(-1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(+980e+3, rel=1e-3) == Scx
    assert pytest.approx(-163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # then for uncracked
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.UNCRACKED)
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(-48.3, rel=1e-2) == ysg
    assert pytest.approx(-1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(+980e+3, rel=1e-3) == Scx
    assert pytest.approx(-163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # then for compressed
    #
    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.COMPRESSED)
    assert pytest.approx(0.00, rel=1e-3) == ycg
    assert pytest.approx(-48.3, rel=1e-2) == ysg
    assert pytest.approx(-1.96, rel=1e-3) == yhg
    assert pytest.approx(500.0e+3, rel=1e-6) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(521.2e+3, rel=1e-4) == Ah
    assert pytest.approx(+980e+3, rel=1e-3) == Scx
    assert pytest.approx(-163.4e+3, rel=1e-3) == Ssx
    assert pytest.approx(    0.0, rel=1e-16) == Shx
    assert pytest.approx(10.42e+9, rel=1e-3) == Jcx
    assert pytest.approx(110.12e+6, rel=1e-3) == Jsx
    assert pytest.approx(11.08e+9, rel=1e-4) == Jhx

    # If I didn't launch solverSLS_NM before for PARTIALIZED I can't have results
    assert rcSection.getAreaProperties(SectionStates.PARTIALIZED) is None

    # Launch withowt forces
    # solverSLS_NM classifies section properly lucly
    # xi is always intended from top. This cause H - 73.46

    rcSection.solverSLS_NM(N=0.0e+3, M=-150e+6, uncracked=False)
    H = rcSection.getDimH()
    assert pytest.approx(H - 73.46, rel=1e-2) == rcSection.xi()

    Ah, Shx, Jhx, Ac, Scx, Jcx, As, Ssx, Jsx, yhg, ycg, ysg = rcSection.getAreaProperties(SectionStates.PARTIALIZED)
    assert pytest.approx(73.46e+3, rel=1e-2) == Ac
    assert pytest.approx(3.53e+3, rel=1e-2) == As
    assert pytest.approx(94.63e+3, rel=1e-2) == Ah
    assert pytest.approx(-213.27, rel=1e-2) == ycg
    assert pytest.approx(-48.3, rel=1e-2) == ysg
    assert pytest.approx(-176.67, rel=1e-2) == yhg
    assert pytest.approx(-2.68e+6, rel=1e-1) == Scx
    assert pytest.approx(+453.1e+3, rel=1e-2) == Ssx
    assert pytest.approx(0.0, abs=1e-6) == Shx
    assert pytest.approx(131.43e+6, rel=1e-1) == Jcx
    assert pytest.approx(1.60e+8, rel=1e-2) == Jsx
    assert pytest.approx(1.091e+9, rel=1e-2) == Jhx



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

"""Module with classes for generate reports various formats.

Main class in module is Reporter. Reporter uses actually only LaTex executable
for build reports.

Classes list:

    1. EnumFBSection
    2. Fragment
    3. FragmentsBuilder
    4. getTemplatesPath
    5. ReportDriverEnum
    6. Reporter
    7. ReportProperties
    8. ReportTemplateEnum
"""

import os
import subprocess
from enum import Enum, unique
from importlib.resources import files
from pathlib import Path
from typing import Any, Dict, List, Literal, Union
import shutil

from jinja2 import Environment, FileSystemLoader
from pydantic import BaseModel

from pycivil.EXAUtils.EXAExceptions import EXAExceptions
from pycivil.EXAUtils import logging as logger


@unique
class ReportDriverEnum(Enum):
    PDFLATEX = 1

@unique
class ReportTemplateEnum(Enum):
    """This Enum describe the main documents level

    Attributes:
        TEX_ENG_CAL: style for official reports
        TEX_KOMA: styled with KOMA scripts LaTex
        TEX_MAIN: styled with main LaTex
    """
    TEX_ENG_CAL = 1
    TEX_KOMA = 2
    TEX_MAIN = 3


def getTemplatesPath() -> Path:
    """Funtion to get templates latex path.

    Returns:
        A Path object.
    """
    return Path(str(files("pycivil") / "templates" / "latex"))


class Fragment:
    """Class that hold fragments as list of strings for use with Reporter.

    This class is a container for complete a template (ex. LaTex template).
    Template are processed after with Reporter class. Will be generated
    automatically a type of file. For example PDF or MD etc.

    """
    def __init__(self, latexTemplatePath: Path = Path()):
        self.__fragment: List[str] = []
        self.__latexTemplatePath: Path = latexTemplatePath
        self.__logoFilePath: Path = Path(self.__latexTemplatePath) / Path("logo.png")

    def set_logo(self, logo_path: Path) -> None:
        self.__logoFilePath = logo_path

    def add_line(self, line: str) -> None:
        self.__fragment.append(line)

    def add_lines(self, lines: List[str]) -> None:
        for ll in lines:
            if ll is not None:
                self.__fragment.append(ll)

    def add_template(self, name: str, placeholders: Dict[str, Any]) -> None:
        file_loader = FileSystemLoader(searchpath=self.__latexTemplatePath)
        env = Environment(
            block_start_string=r"\BLOCK{",
            block_end_string="}",
            variable_start_string=r"\VAR{",
            variable_end_string="}",
            comment_start_string=r"\#{",
            comment_end_string="}",
            line_statement_prefix="%-",
            line_comment_prefix="%#",
            trim_blocks=True,
            autoescape=False,
            loader=file_loader,
        )
        jtemplate = env.get_template(name)
        rendered = jtemplate.render(placeholders, logo_path=self.__logoFilePath.as_posix())
        self.__fragment.append(rendered)

    def add(
        self,
        line: Union[str, None] = None,
        lines: Union[List[str], None] = None,
        templateName: Union[str, None] = None,
        templatePlaceholders: Union[Dict[str, Any], None] = None,
    ) -> int:
        if (
            line is not None
            and templateName is None
            and templatePlaceholders is None
            and lines is None
        ):
            self.add_line(line)
        elif (
            line is None
            and lines is None
            and templateName is not None
            and templatePlaceholders is not None
        ):
            self.add_template(templateName, templatePlaceholders)
        elif (
            line is None
            and lines is not None
            and templateName is None
            and templatePlaceholders is None
        ):
            self.add_lines(lines)
        else:
            raise EXAExceptions("0001", "Wrong args", "")
        return len(self.__fragment)

    def frags(self) -> List[str]:
        return self.__fragment


@unique
class EnumFBSection(Enum):
    SEC_CHAPTER = 1
    SEC_SECTION = 2
    SEC_SUBSECTION = 3
    SEC_SUBSUBSECTION = 4


class FragmentsBuilder:
    def __init__(self):
        self.__fragment: Union[Fragment, None] = None
        self.__fragment_options: Dict[str, Any] = {}

    def _setFragment(self, f: Fragment) -> None:
        self.__fragment = f

    def getFragmentOptions(self) -> Dict[str, Any]:
        return self.__fragment_options

    def _setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        self.__fragment_options = options
        return True

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        for name_opt, value_opt in options.items():
            if name_opt not in self.__fragment_options.keys():
                raise ValueError(f"{name_opt} is not a valid option")
        self.__fragment_options = options
        return True

    def buildFragment(self) -> Fragment:
        raise NotImplementedError("Need to be implemented")

    def fragment(self) -> Union[Fragment, None]:
        return self.__fragment


class ReportProperties(BaseModel):
    project_brief: str = "Brief"
    module_name: str = "Module Name"
    module_version: str = "Module Version"
    report_designer: str = "Designer user name"
    report_date: str = "18/01/1972"
    report_time: str = "01:30"
    report_token: str = "xxxxxxx"
    report_logo: Path | None = None


class Reporter:
    r"""Class that compiles fragments to produce PDF or others.

    This class compiles fragments to produce PDF or others type of files.
    Actually produces PDF using LaTex files by one of LaTeX implementations as
    MikTex (popular implementation for Windows) or TexLive (popular on Linux).

    Typical usage example:

    ```
     reporter = Reporter(latexTemplatePath)
     reporter.linkFragments(ReportDriverEnum.PDFLATEX, ReportTemplateEnum.TEX_ENG_CAL, [frag_title, frag])
     reporter.makePDF(path=str(working_path), fileName="TEX_ENG_CAL")
    ```
    """
    def __init__(self, latexTemplatePath: Path):
        self.__main_file_name: str = ""
        self.__templated: Union[str, None] = ""
        self.__sepForTex: str = "\n"
        self.__opt_makeGlossary: bool = False
        self.__ll: Literal[0, 1, 2, 3] = 3
        self.__latexTemplatePath: Path = latexTemplatePath
        self.__properties: ReportProperties = ReportProperties()

    def setProperties(self, prop: ReportProperties) -> None:
        self.__properties = prop

    def compileDocument(
        self,
        path: str = "",
        fileName: str = "report",
        verbose: bool = False,
        deleteIfExists: bool = True,
    ) -> None:
        """Build a PDF using driver LATEXPDF

        Write before templated in <path> with name <fileName>.
        Extension are automatically added at the end of file name.

        Args:
            deleteIfExists (bool, optional): if True delete file PDF if exists
            verbose (bool, optional):
            path (str, optional): where intermediate file are crated. Defaults to ''.
            fileName (str, optional): input file name. Defaults to 'report'.
        """
        completeFileName = Path(f"{path}/{fileName}")
        completeFileNameTex = Path(f"{completeFileName}.tex")
        completeFileNamePdf = Path(f"{completeFileName}.pdf")

        # Save tex file
        with open(file=completeFileNameTex, mode="w", encoding="utf-8") as f:
            f.write(self.__templated)  # type: ignore

        if deleteIfExists:
            if os.path.exists(completeFileNamePdf):
                os.remove(completeFileNamePdf)
                logger.log(
                    tp="INF", level=self.__ll, msg=f"removed file {completeFileNamePdf}"
                )
            else:
                logger.log(
                    tp="INF",
                    level=self.__ll,
                    msg=f"file do not exists {completeFileNamePdf}",
                )

        # Compile
        os.chdir(path)
        os.listdir()
        #
        # FIRST RUN PDFLATEX
        #
        print("Setted make glossary:")
        print("... 1/4 run first pdflatex")
        self._run_latex(str(completeFileNameTex), verbose)
        if not self.__opt_makeGlossary:
            print("... 2/4 no run makeglossaries")
        else:
            self._make_glossaries(fileName, path, verbose)

        #
        # SECOND RUN PDFLATEX
        #
        print("... 3/4 run pdflatex for tables width adjust")
        self._run_latex(str(completeFileNameTex), verbose)

        #
        # THIRD RUN PDFLATEX
        #
        print("... 4/4 run pdflatex for tables width adjust")
        if self._run_latex(str(completeFileNameTex), verbose):
            print(f"PDF generated as {completeFileNamePdf}")

    def _make_glossaries(self, fileName, path, verbose):
        #
        # SECOND RUN MAKEGLOSSARIES
        #
        print("... 2/4 run makeglossaries")
        print(f"Current working directory: {os.getcwd()}")
        cwdOld = os.getcwd()
        os.chdir(path)
        print(f"for makeglossaries working directory: {os.getcwd()}")
        self._run_command(["makeglossaries", fileName], verbose)
        os.chdir(cwdOld)

    def _run_latex(self, file_name: str, verbose: bool = False) -> bool:
        logger.log("INF", msg=f"File name pdflatex run is in {file_name}", level=self.__ll)

        return self._run_command(["pdflatex", "--no-shell-escape",file_name], verbose)

    @staticmethod
    def _run_command(args: List[str], verbose: bool = False) -> bool:
        # Check if pdflatex is in PATH

        program_path = shutil.which(args[0])
        if shutil.which(args[0]):
            print(f"{program_path} found in PATH {program_path}")
        else:
            print(f"{program_path} NOT found in PATH - need to install")
        print(args)

        if os.name == "nt":
            x = subprocess.run(
                executable=None,
                args=args,
                text=True,
                shell=True,
                capture_output=not verbose)
        else:
            x = subprocess.run(
                executable=args[0],
                args=args[1:len(args)],
                text=True,
                shell=True,
                capture_output=not verbose)

        if x.returncode != 0:
            print("... Exit-code not 0, check result!")
        else:
            print("... Exit-code 0")
        return x.returncode == 0

    def linkFragments(
        self,
        template: ReportTemplateEnum,
        driver: ReportDriverEnum = ReportDriverEnum.PDFLATEX,
        fragments: List[Fragment] | None = None,
        builder: Union[FragmentsBuilder, List[FragmentsBuilder]] | None = None,
        glossary: bool = False,
        main_file_name: str = "",
    ) -> bool:
        """Links fragments adding also a main documents.

        Links fragments adding also a main documents. For example in LaTex
        the linking process add

        ```LaTex
            \\documentclass[11pt, a4paper]{article}
            \\usepackage[latin1]{inputenc}
            ...
            \\begin{document}
            ...
            Fragment line 1
            Fragment line 2
            ...
            Fragment line n
            ...
            \\end{document}
        ```

        Args:
            template (ReportTemplateEnum):
             Enum for main document template.
            driver (ReportDriverEnum, Optional):
             Leave default option.
            fragments (List[Fragment], Optional):
             List of rows to put in main document.
            builder (Union[FragmentsBuilder, List[FragmentsBuilder]], Optional):
             List of fragment builder. This is alternative to fragments
             argument. Default is None.
            glossary (bool, optional): Default is False.
            main_file_name (str, optional): Default is "".

        Returns:
            True on success, False otherwise
        """
        self.__opt_makeGlossary = glossary
        self.__main_file_name = main_file_name
        print("buildPDF: start ...")

        if builder is not None:
            if not isinstance(builder, list):
                builders = [builder]
            else:
                builders = builder

            fragments = []
            for b in builders:
                fragments.append(b.buildFragment())

        united = []
        assert fragments is not None
        for f in fragments:
            if f.frags() is not None:
                united += f.frags()  # type: ignore
            else:
                print("ERR: None fragment")
                print("buildPDF: Quit")
                return False

        if ReportDriverEnum.PDFLATEX != driver:
            print("ERR: driver unknown !!! quit")
            return False

        file_loader = FileSystemLoader(searchpath=self.__latexTemplatePath)
        env = Environment(
            block_start_string="\\BLOCK{",
            block_end_string="}",
            variable_start_string="\\VAR{",
            variable_end_string="}",
            comment_start_string="\\#{",
            comment_end_string="}",
            line_statement_prefix="%-",
            line_comment_prefix="%#",
            trim_blocks=True,
            autoescape=False,
            loader=file_loader,
        )

        print("build with pdflatex ...")

        # LaTex substitution strings
        #
        # 1) Changing "_" with "\_" for latex
        project_brief = self.__properties.project_brief.replace("_", r"\_")

        if (
            template == ReportTemplateEnum.TEX_ENG_CAL
            or template == ReportTemplateEnum.TEX_MAIN
        ):
            print("... template TEX_ENG_CAL")
            if template == ReportTemplateEnum.TEX_ENG_CAL:
                jtemplate = env.get_template("template-eng-cal.tex")
            else:
                jtemplate = env.get_template(self.__main_file_name)

            logoPath = self.__latexTemplatePath / "logo.png"

            # override logo path
            if self.__properties.report_logo is not None:
                logoPath = self.__properties.report_logo

            self.__templated = jtemplate.render(
                place_holder=self.__sepForTex.join(united),
                glossary=self.__opt_makeGlossary,
                logo_path=logoPath.as_posix(),
                project_brief=project_brief,
                module_name=self.__properties.module_name,
                module_version=self.__properties.module_version,
                report_designer=self.__properties.report_designer,
                report_date=self.__properties.report_date,
                report_time=self.__properties.report_time,
                report_token=self.__properties.report_token,
            )

        elif template == ReportTemplateEnum.TEX_KOMA:
            print("... template TEX_KOMA")
            jtemplate = env.get_template("template-koma.tex")
            self.__templated = jtemplate.render(
                glossary=self.__opt_makeGlossary,
                place_holder=self.__sepForTex.join(united),
            )

        else:
            print("ERR: driver unknown !!! quit")
            return False

        return True



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

# VLAD 500.0 300.0 C32/40 B450C 0.85 0KN 193KNm alfa 0 603mm2
import time

from pycivil.EXAStructural import templateRCRect as est

start = time.time()

plotDomain = True

Area = est.RCTemplRectEC2DesignNM(
    name="section_01",
    elementType="beam",
    steel="B450C",
    concrete="C32/40",
    sectionH=500.0,
    sectionW=300.0,
    topRecover=40.0,
    botRecover=40.0,
    N=+0.00 * 1e3 * 1e3,
    M=(+1.04 + 0.94) / 2 * 1e2 * 1e6,
    toll=1e-5,
    toll_bisec=1e-3,
    maxIt=100,
    plot=True,
    alphaTopBot=0,
    factorStart=0,
    factorEnd=1,
    points=100,
    logLevel=1,
)

print(Area)
print("******************")
print("Minimal steel area: %1.1f" % Area[0])
print("******************")

section = est.RCTemplRectEC2(1, "section_01")
section.setDimH(500.0)
section.setDimW(300.0)
section.addSteelArea("MB", 40.0, 572.0)
section.addSteelArea("MT", 40.0, 0.0)

section.setMaterials("C32/40", "B450C")
section.addTensionPoint2d(N=0.0, M=(+1.04 + 0.94) / 2 * 1e2 * 1e6)
section.interactionDomainBuild2d(nbPoints=100, alpha=1.0)
section.interactionDomainPlot2d()

Area = est.RCTemplRectEC2DesignNM(
    name="section_02",
    elementType="beam",
    steel="B450C",
    concrete="C32/40",
    sectionH=500.0,
    sectionW=300.0,
    topRecover=40.0,
    botRecover=40.0,
    N=+0.5 * 1e3 * 1e3,
    M=(+1.91 + 1.72) / 2 * 1e2 * 1e6,
    toll=1e-5,
    toll_bisec=1e-3,
    maxIt=100,
    plot=plotDomain,
    alphaTopBot=0,
    factorStart=0,
    factorEnd=1,
    points=100,
    logLevel=1,
)

print("******************")
print("Minimal steel area: %1.1f" % Area[0])
print("******************")

Area = est.RCTemplRectEC2DesignNM(
    name="section_03",
    elementType="beam",
    steel="B450C",
    concrete="C32/40",
    sectionH=500.0,
    sectionW=300.0,
    topRecover=30.0,
    botRecover=30.0,
    N=+2.0 * 1e3 * 1e3,
    M=+1.91 * 1e2 * 1e6,
    toll=1e-5,
    toll_bisec=1e-3,
    maxIt=100,
    plot=plotDomain,
    alphaTopBot=0.0,
    factorStart=0,
    factorEnd=1,
    points=50,
    logLevel=1,
)
print("******************")
print("Minimal steel area: %1.1f" % Area[0])
print("******************")

Area = est.RCTemplRectEC2DesignNM(
    name="section_04",
    elementType="beam",
    steel="B450C",
    concrete="C32/40",
    sectionH=500.0,
    sectionW=300.0,
    topRecover=30.0,
    botRecover=30.0,
    N=-1.9 * 1e3 * 1e3,
    M=+6.00 * 1e2 * 1e6,
    toll=1e-5,
    toll_bisec=1e-3,
    maxIt=100,
    plot=plotDomain,
    alphaTopBot=0.0,
    factorStart=0,
    factorEnd=1,
    logLevel=1,
)

print("******************")
print("Minimal steel area: %1.1f" % Area[0])
print("******************")

end = time.time()
print(end - start)



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pycivil import EXAGeotechnical as geot
from pycivil import EXAParametric as param

E_top = geot.ModulusYoung()
E_top.setChoose(9)
Et = (E_top.value()[2] + E_top.value()[2]) / 2 * 0.1

E_bot = geot.ModulusYoung()
E_bot.setChoose(2)
Eb = (E_bot.value()[2] + E_bot.value()[2]) / 2 * 0.1

ni_top = geot.RatioPoisson()
ni_top.setChoose(5)
nit = (ni_top.value()[2] + ni_top.value()[2]) / 2

ni_bot = geot.RatioPoisson()
ni_bot.setChoose(3)
nib = (ni_bot.value()[2] + ni_bot.value()[2]) / 2

soilTop = geot.SoilLayer("Terreno superiore - Ghiaia")
soilTop.setEt(Et)
soilTop.setNit(nit)
soilTop.setDry(gamma=20, phi=30, coe=0)
print(soilTop)

soilBot = geot.SoilLayer("Terreno inferiore - Argilla Limosa")
soilBot.setEt(Eb)
soilBot.setNit(nib)
soilBot.setCu(0.05)
print(soilBot)

title = "CENTROPADANA - tombino sottostrada - manufatto I.62 - DN 1000"
circle = param.CircularTubeShape01(
    descr=title, B=1000 + 200 + 300, H=1000 + 100 + 300 + 200, Tw=200, Tt=200, Tb=200
)

circle.assignTopLayer(soilTop)
circle.assignBotLayer(soilBot)
circle.springs()
print(circle)

circle.assignSeismicAction(0.065, 0.2)
circle.setTopCover(7600.0 - 600.0 + 150)
circle.setPaverThickness(600.0, 20.0)

circle.buildFEModel()

circle.model().save("circular", ".msh")
